/* code for "tabread_double" pd class.  This takes two messages: floating-point
numbers, and "rats", and just prints something out for each message. */

#include "m_pd.h"

t_class *tabread_double_class;

    /* the data structure for each copy of "tabread_double".  In this case we
    on;y need pd's obligatory header (of type t_object). */
typedef struct tabread_double
{
	t_object x_obj;
	t_symbol *x_arrayname;
	t_float x_major_double;
} t_tabread_double;

    /* this is called back when tabread_double gets a "float" message (i.e., a
    number.) */
void tabread_double_float(t_tabread_double *x, t_float f)
{
    t_garray *a;
    long npoints;
    t_word *vec;
	const long max_float = 16777216;
	
    if (!(a = (t_garray *)pd_findbyclass(x->x_arrayname, garray_class)))
        pd_error(x, "%s: no such array", x->x_arrayname->s_name);
    else if (!garray_getfloatwords(a, &npoints, &vec))
        pd_error(x, "%s: bad template for tabread", x->x_arrayname->s_name);
    else if ((long) f >16777216)
		post(x, "%s: %i is too big, use the second inlet", x->x_arrayname->s_name,(long) f);
	else
    {
        long n = (long)f+(long)x->x_major_double*max_float;
        if (n < 0) n = 0;
        else if (n >= npoints) n = npoints - 1;
		//post("nindex: %30i", n);
        outlet_float(x->x_obj.ob_outlet, (npoints ? vec[n].w_float : 0));
    }
}

void tabread_double_set(t_tabread_double *x, t_symbol *s)
{
    x->x_arrayname = s;
}

void *tabread_double_new(t_symbol *s)
{
    t_tabread_double *x = (t_tabread_double *)pd_new(tabread_double_class);
    x->x_arrayname = s;
	floatinlet_new(&x->x_obj, &x->x_major_double);
    outlet_new(&x->x_obj, &s_float);
    return (x);
}

void tabread_double_setup(void)
{
    tabread_double_class = class_new(gensym("tabread_double"), (t_newmethod)tabread_double_new,
							  0, sizeof(t_tabread_double), 0, A_DEFSYM, 0);
    class_addfloat(tabread_double_class, (t_method)tabread_double_float);
    class_addmethod(tabread_double_class, (t_method)tabread_double_set, gensym("set"),
					A_SYMBOL, 0);
}

